package handler

import (
	"context"

	"google.golang.org/grpc/metadata"

	"git.loafle.net/commons_go/logging"
	"git.loafle.net/overflow/overflow_gateway_web/grpc"
	"git.loafle.net/overflow/overflow_gateway_web/server"
	ogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/protocol/jsonrpc"
	"go.uber.org/zap"
)

func newWebJSONRpcHandler(ctx context.Context) ogw.ProtocolHandler {
	h := &webJSONRpcHandlers{
		ctx:    ctx,
		logger: logging.WithContext(ctx),
	}

	p := jsonrpc.New(ctx, h)

	return p
}

type webJSONRpcHandlers struct {
	jsonrpc.JSONRpcHandlers
	ctx    context.Context
	logger *zap.Logger
}

func (h *webJSONRpcHandlers) OnRequest(soc ogw.Socket, method string, params []string) (interface{}, error) {
	h.logger.Info("OnRequest",
		zap.String("path", soc.Path()),
		zap.String("method", method),
		zap.Any("params", params),
	)

	uid := server.GetUID(soc.Path(), soc)

	md := metadata.Pairs("email", uid)
	ctx := metadata.NewOutgoingContext(context.Background(), md)

	r, err := grpc.Exec(ctx, method, params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (h *webJSONRpcHandlers) OnNotify(soc ogw.Socket, method string, params []string) error {

	return nil
}
