package handler

import (
	"context"
	"time"

	"git.loafle.net/commons_go/config"
	"git.loafle.net/commons_go/logging"
	ogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"go.uber.org/zap"
)

type webSocketHandlers struct {
	ogw.SocketHandlers
	ctx    context.Context
	logger *zap.Logger
	cfg    config.Configurator
}

func NewWebSocketHandler(ctx context.Context) ogw.SocketHandler {
	h := &webSocketHandlers{
		ctx:    ctx,
		logger: logging.WithContext(ctx),
	}
	h.cfg = config.Sub("handlers.web")
	h.MaxMessageSize = h.cfg.GetInt64("socket.MaxMessageSize")
	h.WriteTimeout = h.cfg.GetDuration("socket.WriteTimeout") * time.Second
	h.ReadTimeout = h.cfg.GetDuration("socket.ReadTimeout") * time.Second
	h.PongTimeout = h.cfg.GetDuration("socket.PongTimeout") * time.Second
	h.PingTimeout = h.cfg.GetDuration("socket.PingTimeout") * time.Second
	h.PingPeriod = h.cfg.GetDuration("socket.PingPeriod") * time.Second
	h.BinaryMessage = h.cfg.GetBool("socket.BinaryMessage")

	ph := newWebJSONRpcHandler(ctx)

	h.Protocol = ph

	return h
}
